@@ -4,6 +4,9 @@ use cogo::std::time::time::Time;
 fn main() {
     let mut t = Time::now();
     println!("{}", t);
+    println!("{}", t.unix());
+    println!("{}", t.unix_nano());
+
     let js = serde_json::to_string(&t).unwrap();
     println!("{}", js);
     let from_js = serde_json::from_str::<Time>(&js).unwrap();
