@@ -1,4 +1,5 @@
 use std::fmt::{self, Debug, Display};
+use std::io::ErrorKind::UnexpectedEof;
 
 pub type Result<T> = std::result::Result<T, Error>;
 
@@ -14,6 +15,7 @@ impl Error {
 }
 
 ///new error
+#[inline]
 pub fn new(text: String) -> Error {
     Error {
         inner: text
@@ -33,6 +35,9 @@ impl ToString for Error {
 impl From<std::io::Error> for Error {
     #[inline]
     fn from(err: std::io::Error) -> Self {
+        if err.kind().eq(&UnexpectedEof) {
+            return new("EOF".to_string());
+        }
         new(err.to_string())
     }
 }
@@ -66,3 +71,15 @@ impl From<&Box<dyn std::error::Error>> for Error {
         return new(arg.to_string());
     }
 }
+
+impl From<time::error::InvalidFormatDescription> for Error{
+    fn from(arg: time::error::InvalidFormatDescription) -> Self {
+        return new(arg.to_string());
+    }
+}
+
+impl From<time::error::Parse> for Error {
+    fn from(arg: time::error::Parse) -> Self {
+        return new(arg.to_string());
+    }
+}
\ No newline at end of file
