@@ -6,7 +6,7 @@
 //! would not see that the same data any more
 
 use std::fmt;
-use std::sync::atomic::{AtomicUsize, Ordering};
+use std::sync::atomic::{AtomicBool, AtomicUsize, Ordering};
 use std::sync::mpsc::{RecvError, RecvTimeoutError, SendError, TryRecvError};
 use std::sync::Arc;
 use std::time::Duration;
@@ -31,7 +31,7 @@ pub fn unbounded<T>() -> (Sender<T>, Receiver<T>) {
 
 /// Create a bounded channel
 pub fn bounded<T>(buf: usize) -> (Sender<T>, Receiver<T>) {
-    let a = Arc::new(InnerQueue::new_buffer(buf));
+    let a = Arc::new(MPMCBuffer::new_buffer(buf));
     (Sender::new(a.clone()), Receiver::new(a))
 }
 
@@ -56,47 +56,47 @@ macro_rules! chan {
 
 
 /// /////////////////////////////////////////////////////////////////////////////
-/// InnerQueue
+/// MPMCBuffer
 /// /////////////////////////////////////////////////////////////////////////////
-struct InnerQueue<T> {
-    queue: SegQueue<T>,
+struct MPMCBuffer<T> {
+    buffer: SegQueue<T>,
+    // chan buffer length limit. Exceeding this limit will be wait.
+    buffer_limit: usize,
     // thread/coroutine for wake up
     wake_recv: Semphore,
     // thread/coroutine for wake up
     wake_sender: Semphore,
-    // chan buffer length
-    buffer_len: usize,
-    // The number of tx channels which are currently using this queue.
-    tx_ports: AtomicUsize,
-    // if rx is dropped
-    rx_ports: AtomicUsize,
+    // The number of sender channels which are currently using this queue.
+    sender_num: AtomicUsize,
+    // The number of receiver
+    receiver_num: AtomicUsize,
 }
 
-impl<T> InnerQueue<T> {
+impl<T> MPMCBuffer<T> {
     /// default is an Unbounded buffer
-    pub fn new() -> InnerQueue<T> {
-        InnerQueue::new_buffer(usize::MAX)
+    pub fn new() -> MPMCBuffer<T> {
+        MPMCBuffer::new_buffer(usize::MAX)
     }
 
     /// have buffer channel. If the buffered message exceeds the limit, the sender blocks until the message is consumed
-    pub fn new_buffer(mut buffer: usize) -> InnerQueue<T> {
-        InnerQueue {
-            queue: SegQueue::new(),
+    pub fn new_buffer(mut buffer: usize) -> MPMCBuffer<T> {
+        MPMCBuffer {
+            buffer: SegQueue::new(),
             wake_recv: Semphore::new(0),
             wake_sender: Semphore::new(0),
-            buffer_len: buffer,
-            tx_ports: AtomicUsize::new(1),
-            rx_ports: AtomicUsize::new(1),
+            buffer_limit: buffer,
+            sender_num: AtomicUsize::new(1),
+            receiver_num: AtomicUsize::new(1),
         }
     }
 
     pub fn send(&self, t: T) -> Result<(), SendError<T>> {
-        if self.rx_ports.load(Ordering::Acquire) == 0 {
+        if self.receiver_num.load(Ordering::Acquire) == 0 {
             return Err(SendError(t));
         }
-        self.queue.push(t);
+        self.buffer.push(t);
         self.wake_recv.post();
-        if self.queue.len() > self.buffer_len {
+        if self.buffer.len() > self.buffer_limit {
             self.wake_sender.wait();
         }
         Ok(())
@@ -105,7 +105,7 @@ impl<T> InnerQueue<T> {
     /// wake one sender
     #[inline]
     fn wake_sender(&self) {
-        if (self.queue.len() + 1) > self.buffer_len {
+        if (self.buffer.len() + 1) > self.buffer_limit {
             self.wake_sender.post();
         }
     }
@@ -126,12 +126,12 @@ impl<T> InnerQueue<T> {
             }
         }
 
-        match self.queue.pop() {
+        match self.buffer.pop() {
             Some(data) => {
                 self.wake_sender();
                 Ok(data)
             }
-            None => match self.tx_ports.load(Ordering::Acquire) {
+            None => match self.sender_num.load(Ordering::Acquire) {
                 0 => Err(RecvTimeoutError::Disconnected),
                 _n => unreachable!("mpmc recv found no data"),
             },
@@ -140,72 +140,80 @@ impl<T> InnerQueue<T> {
 
     pub fn try_recv(&self) -> Result<T, TryRecvError> {
         if !self.wake_recv.try_wait() {
-            return match self.tx_ports.load(Ordering::Acquire) {
+            return match self.sender_num.load(Ordering::Acquire) {
                 0 => Err(TryRecvError::Disconnected),
                 _ => Err(TryRecvError::Empty),
             };
         }
 
-        match self.queue.pop() {
+        match self.buffer.pop() {
             Some(data) => {
                 self.wake_sender();
                 Ok(data)
             }
-            None => match self.tx_ports.load(Ordering::Acquire) {
+            None => match self.sender_num.load(Ordering::Acquire) {
                 0 => Err(TryRecvError::Disconnected),
                 _ => unreachable!("mpmc try_recv found no data"),
             },
         }
     }
 
-    pub fn clone_tx(&self) {
-        self.tx_ports.fetch_add(1, Ordering::SeqCst);
+    pub fn clone_send(&self) {
+        self.sender_num.fetch_add(1, Ordering::SeqCst);
     }
 
-    pub fn drop_tx(&self) {
-        match self.tx_ports.fetch_sub(1, Ordering::SeqCst) {
+    pub fn drop_send(&self) {
+        match self.sender_num.fetch_sub(1, Ordering::SeqCst) {
             1 => {
-                // there is no tx port any more
-                // should tell all the waited rx to come back
+                // there is no send_ports any more
+                // should tell all the waited recv to come back
                 while self.wake_recv.get_value() == 0 {
                     self.wake_recv.post();
                 }
             }
             n if n > 1 => {}
-            n => panic!("bad number of tx_ports left {}", n),
+            n => panic!("bad number of send_ports left {}", n),
         }
     }
 
-    pub fn clone_rx(&self) {
-        self.rx_ports.fetch_add(1, Ordering::SeqCst);
+    pub fn clone_recv(&self) {
+        self.receiver_num.fetch_add(1, Ordering::SeqCst);
     }
 
-    pub fn drop_rx(&self) {
-        match self.rx_ports.fetch_sub(1, Ordering::SeqCst) {
+    pub fn drop_recv(&self) {
+        match self.receiver_num.fetch_sub(1, Ordering::SeqCst) {
             1 => {
                 // there is no receiver any more, clear the data
-                while self.queue.pop().is_some() {}
+                while self.buffer.pop().is_some() {}
             }
             n if n > 1 => {}
-            n => panic!("bad number of rx_ports left {}", n),
+            n => panic!("bad number of recv_ports left {}", n),
         }
     }
 
     /// return remain msg len
     pub fn remain(&self) -> usize {
-        self.queue.len()
+        self.buffer.len()
+    }
+
+    pub fn sender_num(&self) -> usize {
+        self.sender_num.load(Ordering::SeqCst)
+    }
+
+    pub fn receiver_num(&self) -> usize {
+        self.receiver_num.load(Ordering::SeqCst)
     }
 }
 
-impl<T> Drop for InnerQueue<T> {
+impl<T> Drop for MPMCBuffer<T> {
     fn drop(&mut self) {
-        assert_eq!(self.tx_ports.load(Ordering::Acquire), 0);
-        assert_eq!(self.rx_ports.load(Ordering::Acquire), 0);
+        assert_eq!(self.sender_num.load(Ordering::Acquire), 0);
+        assert_eq!(self.receiver_num.load(Ordering::Acquire), 0);
     }
 }
 
 pub struct Receiver<T> {
-    inner: Arc<InnerQueue<T>>,
+    inner: Arc<MPMCBuffer<T>>,
 }
 
 impl<T> Receiver<T> {
@@ -213,25 +221,35 @@ impl<T> Receiver<T> {
     pub fn remain(&self) -> usize {
         self.inner.remain()
     }
+
+    /// Number of channel senders
+    pub fn sender_num(&self) -> usize {
+        self.inner.sender_num()
+    }
+
+    /// Number of channel receiver
+    pub fn receiver_num(&self) -> usize {
+        self.inner.receiver_num()
+    }
 }
 
 unsafe impl<T: Send> Send for Receiver<T> {}
 // impl<T> !Sync for Receiver<T> {}
 
 pub struct Iter<'a, T: 'a> {
-    rx: &'a Receiver<T>,
+    inner: &'a Receiver<T>,
 }
 
 pub struct TryIter<'a, T: 'a> {
-    rx: &'a Receiver<T>,
+    inner: &'a Receiver<T>,
 }
 
 pub struct IntoIter<T> {
-    rx: Receiver<T>,
+    inner: Receiver<T>,
 }
 
 pub struct Sender<T> {
-    inner: Arc<InnerQueue<T>>,
+    inner: Arc<MPMCBuffer<T>>,
 }
 
 impl<T> Sender<T> {
@@ -239,6 +257,16 @@ impl<T> Sender<T> {
     pub fn remain(&self) -> usize {
         self.inner.remain()
     }
+
+    /// Number of channel senders
+    pub fn sender_num(&self) -> usize {
+        self.inner.sender_num()
+    }
+
+    /// Number of channel receiver
+    pub fn receiver_num(&self) -> usize {
+        self.inner.receiver_num()
+    }
 }
 
 unsafe impl<T: Send> Send for Sender<T> {}
@@ -250,7 +278,7 @@ unsafe impl<T: Send> Send for Sender<T> {}
 /// /////////////////////////////////////////////////////////////////////////////
 
 impl<T> Sender<T> {
-    fn new(inner: Arc<InnerQueue<T>>) -> Sender<T> {
+    fn new(inner: Arc<MPMCBuffer<T>>) -> Sender<T> {
         Sender { inner }
     }
 
@@ -266,14 +294,14 @@ impl<T> Sender<T> {
 
 impl<T> Clone for Sender<T> {
     fn clone(&self) -> Sender<T> {
-        self.inner.clone_tx();
+        self.inner.clone_send();
         Sender::new(self.inner.clone())
     }
 }
 
 impl<T> Drop for Sender<T> {
     fn drop(&mut self) {
-        self.inner.drop_tx();
+        self.inner.drop_send();
     }
 }
 
@@ -288,7 +316,7 @@ impl<T> fmt::Debug for Sender<T> {
 /// /////////////////////////////////////////////////////////////////////////////
 
 impl<T> Receiver<T> {
-    fn new(inner: Arc<InnerQueue<T>>) -> Receiver<T> {
+    fn new(inner: Arc<MPMCBuffer<T>>) -> Receiver<T> {
         Receiver { inner }
     }
 
@@ -308,11 +336,11 @@ impl<T> Receiver<T> {
     }
 
     pub fn iter(&self) -> Iter<T> {
-        Iter { rx: self }
+        Iter { inner: self }
     }
 
     pub fn try_iter(&self) -> TryIter<T> {
-        TryIter { rx: self }
+        TryIter { inner: self }
     }
 }
 
@@ -320,7 +348,7 @@ impl<'a, T> Iterator for Iter<'a, T> {
     type Item = T;
 
     fn next(&mut self) -> Option<T> {
-        self.rx.recv().ok()
+        self.inner.recv().ok()
     }
 }
 
@@ -328,7 +356,7 @@ impl<'a, T> Iterator for TryIter<'a, T> {
     type Item = T;
 
     fn next(&mut self) -> Option<T> {
-        self.rx.try_recv().ok()
+        self.inner.try_recv().ok()
     }
 }
 
@@ -344,7 +372,7 @@ impl<'a, T> IntoIterator for &'a Receiver<T> {
 impl<T> Iterator for IntoIter<T> {
     type Item = T;
     fn next(&mut self) -> Option<T> {
-        self.rx.recv().ok()
+        self.inner.recv().ok()
     }
 }
 
@@ -353,20 +381,20 @@ impl<T> IntoIterator for Receiver<T> {
     type IntoIter = IntoIter<T>;
 
     fn into_iter(self) -> IntoIter<T> {
-        IntoIter { rx: self }
+        IntoIter { inner: self }
     }
 }
 
 impl<T> Clone for Receiver<T> {
     fn clone(&self) -> Receiver<T> {
-        self.inner.clone_rx();
+        self.inner.clone_recv();
         Receiver::new(self.inner.clone())
     }
 }
 
 impl<T> Drop for Receiver<T> {
     fn drop(&mut self) {
-        self.inner.drop_rx();
+        self.inner.drop_recv();
     }
 }
 
